      PROGRAM MEASERR
C **********************************************************************
C *              CALIFORNIA DISTRICT PROGRAMS (USGS - WRD)             *
C **********************************************************************
C
C **********************************************************************
C *  PROGRAM: MEASERR - MEASUREMENT UNCERTAINTY ERROR                  *
C *   AUTHOR: dkyancey@s101dcascr.wr.usgs.gov                          *
C *                                                                    *
C *   SYSTEM:                                       RELEASE: 93.1.0    *
C * LANGUAGE: FORTRAN 77                               DATE: 3/93      *
C **********************************************************************
C
C **********************************************************************
C * PURPOSE:                                                           *
C * CALCULATE WATER DISCHARGE MEASUREMENT % UNCERTAINTY AS DESCRIBED   *
C * IN USGS OPEN-FILE REPORT 92-144 (SAUER & MEYER)                    *
C **********************************************************************
C
C **********************************************************************
C *                           MODIFICATIONS                            *
C * DESCRIPTION:                       PROGRAMMER:           DATE:     *
C * Modified for use on Data General   R. S. Regan  Release: V930501   *
C * and PC's                                           Date: 93/04/19  *
C *   Used Fortran static analysers to clean-up code.  Below is a list *
C *   of problems indentified and corrected or comment.  Diagnostics   *
C *   were generated by MAT (Maintainability Analysis Tool).  Also,    *
C *   enforced consistent spacing, formating, and statement labeling   *
C *   of code.  To use the code on Data General AViiON and PC's also   *
C *   include the TNOUA replacement routine as shown below.  For use   *
C *   on PRIME computers modify the ERSMSG routine (last routine) as   *
C *   indicated in the routine.  Added strings to allow indentification*
C *   of executable using UNIX  what command and RCS (GNU Revision     *
C *   Control System) ident command.  Use PRINT command for all screen *
C *   operations and WRITE command for all file (internal and external)*
C *   operations.  Values of measurement variables are retained from   *
C *   values entered for a previous measurement.  Fixed problem where  *
C *   NVERT was read using I4 format and declared as REAL, now NVERT   *
C *   is and INTEGER and RNVERT holds the corresponding real value.    *
C *   Variable MACCY declared 10 bytes needed to be 12.  Use of PRINT  *
C *   statement with concatenated strings caused an internal error on  *
C *   the DG's as a work around these strings were first assigned to a *
C *   variable and then printed or printed using a format without      *
C *   concatenation.  The clear message area routine was changed to    *
C *   blank out lines 18-22 with a blank record instead of using the   *
C *   ^[J command as the escape sequence wiped out all of the screen   *
C *   on the PC.  The old ERSMSG is used on the PRIME as the PRIME     *
C *   deletes trailing blanks.  The OUTFIL variable increased from 9   *
C *   characters to 12 (measer##.out).  The NCRLF routine was moved    *
C *   near to the end of the file (second to last routine) and also    *
C *   puts a leading blank character of each string to be output.      *
C *   Changed function VALSTR to read from internal file as formatted  *
C *   input since F77 expects internal files to be formatted, value    *
C *   read as Fn.0, where n is the length of the character string      *
C *   passed to VALSTR.                                                *
C Count
C
C      Interface Check:
C   1   Subprogram (or entry pt) is referenced but not defined (TNOUA, INTS)
C          Note, INTS not part of ANSI F77, TNOUA Prime routine
C   2   Called subroutine or function sets argument but should not
C          /* GETSTR (pass constant to routine that can be reset--arg 1)
C          /* in ERRMSG calls to MSGPOS arg 1 & 2
C   2   This module does not properly use an argument passed to it earlier
C          /* MSGPOS (pass constant to routine that can be reset--arg 1 & 2)
C          /* ERSMSG (pass constant to routine that can be reset--arg 1)
C
C      Style Error:
C   1   Self-reference, possibly leading to endless looping, occurs at
C       label 5 (main)
C
C      F77 Standard Violation:
C   1   Branching anomaly
C          /* in main at statement labels 20, 25, 30, 35, and 50 --
C             branch in If/Else-If block (note original statement labels)
C
C      Poor Usage:
C   2   Local variable is defined but not used       /* in main TYPMES, in
C          VALSTR LSTR
C   1   Local variable is set but not referenced     /* in ERRMSG CTNU
C
C      Notice:
C   1   Local variable is referenced and passed as an argument but
C       possibly not set   /* in main QUIT
C   2   Local variable is set and passed as an argument but not otherwise
C       referenced   /* in main PMSG, in NCRLF LSTR
C *                                                                    *
C **********************************************************************
C
C **********************************************************************
C *  EXTERNAL SUBROUTINES/FUNCTIONS CALLED  /* with comments           *
C **********************************************************************
C TNOUA  /* this is a PRIME system routine for other platforms the following
C        /* routine can be used if VAX F77 extensions are supported.
C
C     SUBROUTINE TNOUA(BUF,LEN)
C     CHARACTER*(*) BUF
C     INTEGER*2 LEN
C     INTEGER N
C     N = LEN
C     PRINT '(A,$)', BUF(1:N)
C     RETURN
C     END
C
C **********************************************************************
C *  INTRINSIC FUNCTIONS                    /* with comments           *
C **********************************************************************
C    SQRT
C    IABS
C    LEN
C
C **********************************************************************
C *  VARIABLE DECLARATIONS                  /* with comments           *
C **********************************************************************
C
      CHARACTER STANUM*15, STANAM*40, PRVSTA*15, NUMPTS*1, METER*6, 
     $          TYPE*1, BEDTYP*1, OWFILE*80, OUTFIL*12, MACCY*12
      CHARACTER AMESNO*5, AMETER*1, AMEANV*8, AWIDTH*8, AAREA*8, AQ*10, 
     $          ATIOBS*3, AHORIZ*1, ANVERT*4, AOK*1, FNOTE*1
      CHARACTER PMSG*80, VERSN*8
      INTEGER TIOBS, SH, LENSTR, IUN, OUN, IOS, SEQNUM, NVERT
      REAL MEANV, WIDTH, AREA, SBSDSV, DM, SD, SI, ST, SS, SV, SQ
      REAL VALSTR, RNVERT
      LOGICAL OUTWAT, HORIZA, VWARN, QUIT, REDO, NUMBER, WOUT, EXISTS, 
     $        HEAD
C
C--SET CONSTANT FOR SYSTEMATIC ERRORS (Sb,Sd,Sv)
      PARAMETER (SBSDSV=0.75)
C
C--I/O PARAMETERS
      PARAMETER (IUN=60)
      PARAMETER (OUN=61)
C
C--DEFINE SOME ANSI CONTROL CHARACTERS
      CHARACTER CSI*2
      CHARACTER REV*4, BOLD*4, OFF*4
      CSI = CHAR(27)//'['
      REV = CSI//'7m'
      BOLD = CSI//'1m'
      OFF = CSI//'0m'
      QUIT = .FALSE.
C     set string for use if RCS ident command
      PMSG = 
     $'$Id: measerr.f,v 2.5 1994/04/19 21:37:24 rsregan Exp rsregan $'
      VERSN = PMSG(24:31)
C     set string for use with UNIX what command
      PMSG = 
     $ '@(#)MEASERR - Discharge Measurement Error Program'
      PMSG = '@(#)MEASERR - USGS OFR 92-144, V.B. Sauer and R.W. Meyer'
      PMSG = '@(#)MEASERR - Contact: h2osoft@usgs.gov'
      PMSG = '@(#)MEASERR - Version: 2.5x 1994/04/19'
C
C--OPEN OUTPUT FILE
      SEQNUM = 1
C     file name limited to 12 characters to be compatible with DOS
 100  CONTINUE
      WRITE (OUTFIL,9010) 'measer', SEQNUM, '.out'
      INQUIRE (FILE=OUTFIL,EXIST=EXISTS,IOSTAT=IOS)
      IF (EXISTS .OR. IOS.NE.0) THEN
        SEQNUM = SEQNUM + 1
        IF (SEQNUM.LT.99) GOTO 100
        PRINT *, '** Too many ''measer##.out'' files exist.'
        PRINT *, '   You must rename or delete some of these files.'
        STOP
      ENDIF
      OPEN (OUN,FILE=OUTFIL,STATUS='NEW',IOSTAT=IOS)
      IF (IOS.NE.0) THEN
        PRINT *, '** Unable to open output file: ', OUTFIL
        STOP
      ENDIF
C
C--INITIAL TITLE SCREEN
      CALL CLS()
      PRINT 9005, REV, OFF, VERSN, BOLD, 'QUIT', OFF
C
C--GET DATA FROM ADAPS "OUTWAT" FILE, IF EXISTS
      WOUT = .FALSE.
      OWFILE = ' '
      CALL NCRLF('enter the filename, otherwise press <CR>: ')
      READ (*,'(A)') OWFILE
      IF (INDEX('QUIT quit Quit ',OWFILE(1:5)).GT.0) GOTO 1500
      OUTWAT = .FALSE.
      IF (OWFILE.NE.' ') THEN
        OPEN (IUN,FILE=OWFILE,STATUS='OLD',IOSTAT=IOS)
        IF (IOS.NE.0) THEN
          PRINT *, '** Unable to open OUTWAT file: ', OWFILE
          GOTO 1500
        ENDIF
        OUTWAT = .TRUE.
      ENDIF
C
C--INIT
      STANUM = ' '
      STANAM = ' '
      PRVSTA = ' '
      VWARN = .FALSE.
      AMETER = 'A'
      NUMPTS = '1'
      TYPE = 'W'
      BEDTYP = 'A'
      ATIOBS = '42'
      AHORIZ = 'N'
      AMEANV = ' '
      AWIDTH = ' '
      AMESNO = ' '
      AAREA = ' '
      ANVERT = ' '
C
C--TOP OF LOOP TO PROCESS EACH MEASUREMENT
 200  CONTINUE
      FNOTE = ' '
      HEAD = .TRUE.
C
      IF (OUTWAT) THEN
 250    CONTINUE
        READ (IUN,9015,END=1500,ERR=250) STANUM, AMESNO, AWIDTH, AAREA, 
     $                                   AMEANV, AQ, ANVERT
        IF (.NOT.NUMBER(STANUM)) GOTO 250
        REDO = .FALSE.
      ELSE
        REDO = .TRUE.
      ENDIF
      CALL CLS()
C
C--PUT ALL INPUT INFO ON SCREEN FIRST
      CALL MSGPOS(2,2,'Station Number..........: '//STANUM)
      CALL MSGPOS(3,2,'Station Name............: '//STANAM)
      CALL MSGPOS(4,2,'Measurement No..........: '//AMESNO)
      CALL MSGPOS(5,2,'Channel Width (ft)......: '//AWIDTH)
      CALL MSGPOS(6,2,'X-Section Area (sq.ft)..: '//AAREA)
      CALL MSGPOS(7,2,'Mean Velocity (ft/sec)..: '//AMEANV)
      CALL MSGPOS(8,2,'Meter Type..............: '//AMETER)
      CALL MSGPOS(9,2,'Horizontal Angles.......: '//AHORIZ)
      CALL MSGPOS(10,2,'Ave. Observation Time...: '//ATIOBS)
      CALL MSGPOS(11,2,'No. of Vertical Sections: '//ANVERT)
      CALL MSGPOS(12,2,'No. Points in Vertical..: '//NUMPTS)
      CALL MSGPOS(13,2,'Measurement Type........: '//TYPE)
      CALL MSGPOS(14,2,'Channel Bed Type........: '//BEDTYP)
      CALL MSGPOS(16,1,
     $'-----------------------------------------------------------------
     $---------------')
      CALL MSGPOS(18,4,'Answer all questions. Default answers may'//
     $            ' be accepted by pressing <CR>.')
      CALL MSGPOS(19,4,'If you make a mistake, continue answering the '
     $            //'rest of the questions. When')
      CALL MSGPOS(20,4,'finished, you will have an opportunity to make '
     $            //'whatever corrections are')
      CALL MSGPOS(21,4,'necessary before the data is accepted.')
C
C--GET REQUIRED DATA
 300  CONTINUE
      IF (STANUM.EQ.' ' .OR. REDO) THEN
 350    CONTINUE
        CALL GETSTR(2,'Station Number..........: ',-15,STANUM,
     $              '[Enter 8- or 15-digit station ID]',QUIT)
        IF (QUIT) GOTO 1500
        IF (STANUM.EQ.' ') GOTO 350
        IF (LENSTR(STANUM).NE.8 .AND. LENSTR(STANUM).NE.15) THEN
          CALL ERRMSG('STATION NUMBER MUST BE EITHER 8 OR 15 DIGITS',
     $                .FALSE.)
          STANUM = ' '
          GOTO 350
        ENDIF
      ENDIF
C
C--IF MORE FOR SAME STATION
      IF (STANUM.EQ.PRVSTA) THEN
        CALL MSGPOS(3,2,'Station Name............: '//STANAM)
      ELSE
        PRVSTA = STANUM
        CALL GETSTR(3,'Station Name............: ',-40,STANAM,
     $              '[Optional]',QUIT)
        IF (QUIT) GOTO 1500
        CALL UPCASE(STANAM)
        IF (HEAD) THEN
          HEAD = .FALSE.
          WRITE (OUN,9020) STANUM(:LENSTR(STANUM)), STANAM
          IF (OUTWAT) THEN
            WRITE (OUN,9025)
          ELSE
            WRITE (OUN,9030)
          ENDIF
        ENDIF
      ENDIF
C
      IF (AMESNO.EQ.' ' .OR. REDO) THEN
 400    CONTINUE
        CALL GETSTR(4,'Measurement No..........: ',-5,AMESNO,' ',QUIT)
        IF (AMESNO.EQ.' ') GOTO 400
        IF (QUIT) GOTO 1500
      ENDIF
C
      WIDTH = VALSTR(AWIDTH)
      IF (AWIDTH.EQ.' ' .OR. WIDTH.LE.0.0 .OR. REDO) THEN
 450    CONTINUE
        CALL GETSTR(5,'Channel Width (ft)......: ',-6,AWIDTH,' ',QUIT)
        IF (QUIT) GOTO 1500
        WIDTH = VALSTR(AWIDTH)
        IF (AWIDTH.EQ.' ' .OR. WIDTH.LE.0.0) GOTO 450
      ENDIF
C
      AREA = VALSTR(AAREA)
      IF (AAREA.EQ.' ' .OR. AREA.LE.0.0 .OR. REDO) THEN
 500    CONTINUE
        CALL GETSTR(6,'X-Section Area (sq.ft)..: ',-9,AAREA,' ',QUIT)
        IF (QUIT) GOTO 1500
        AREA = VALSTR(AAREA)
        IF (AAREA.EQ.' ' .OR. AREA.LE.0.0) GOTO 500
      ENDIF
C
 600  CONTINUE
      MEANV = VALSTR(AMEANV)
      IF (AMEANV.EQ.' ' .OR. MEANV.LE.0.0 .OR. REDO) THEN
 650    CONTINUE
        CALL GETSTR(7,'Mean Velocity (ft/sec)..: ',-5,AMEANV,
     $              '[Allowable range is 0.1 to 50]',QUIT)
        IF (QUIT) GOTO 1500
        MEANV = VALSTR(AMEANV)
        IF (AMEANV.EQ.' ' .OR. MEANV.LT.0.0) GOTO 650
      ENDIF
      IF (MEANV.LT..1 .OR. MEANV.GT.50.) THEN
        CALL ERRMSG('MEAN VELOCITY IS OUTSIDE COMPUTATION LIMITS',
     $              .FALSE.)
        AMEANV = ' '
        GOTO 600
      ENDIF
C
      CALL ERSMSG()
      CALL MSGPOS(18,25,BOLD//'A) standard AA meter')
      CALL MSGPOS(19,25,'P) standard Pygmy meter')
      CALL MSGPOS(20,25,'I) Individually rated pygmy meter'//OFF)
 700  CONTINUE
      CALL GETSTR(8,'Meter Type..............: ',1,AMETER,
     $            '[Enter: A, P, or I]',QUIT)
      IF (QUIT) GOTO 1500
      CALL UPCASE(AMETER)
      IF (INDEX('API',AMETER).EQ.0) GOTO 700
      IF (AMETER.EQ.'A') THEN
        METER = 'AASTD '
      ELSEIF (AMETER.EQ.'P') THEN
        METER = 'PYGSTD'
      ELSE
        METER = 'PYGIND'
      ENDIF
      CALL ERSMSG()
C
      CALL MSGPOS(18,21,
     $            BOLD//'NOTE: Answer Yes only if horizontal angles')
      CALL MSGPOS(19,21,'were predominant in the measurement.'//OFF)
 800  CONTINUE
      CALL GETSTR(9,'Horizontal Angles.......: ',1,AHORIZ,
     $            '[Please answer (Y)es or (N)o]',QUIT)
      IF (QUIT) GOTO 1500
      IF (INDEX('Yy',AHORIZ).GT.0) THEN
        HORIZA = .TRUE.
      ELSEIF (INDEX('Nn',AHORIZ).GT.0) THEN
        HORIZA = .FALSE.
      ELSE
        GOTO 800
      ENDIF
      CALL ERSMSG()
C
      CALL MSGPOS(18,19,
     $            BOLD//'NOTE: A general value determined from a quick')
      CALL MSGPOS(19,19,'overview of the measurement is sufficient.'//
     $            OFF)
 900  CONTINUE
      CALL GETSTR(10,'Ave. Observation Time...: ',-3,ATIOBS,' ',QUIT)
      IF (QUIT) GOTO 1500
      READ (ATIOBS,'(I3)',IOSTAT=IOS) TIOBS
      IF (IOS.NE.0 .OR. ATIOBS.EQ.' ') GOTO 900
      CALL ERSMSG()
C
      READ (ANVERT,'(I4)',IOSTAT=IOS) NVERT
      IF (IOS.NE.0 .OR. ANVERT.EQ.' ' .OR. NVERT.LT.1 .OR. REDO) THEN
 950    CONTINUE
        CALL GETSTR(11,'No. of Vertical Sections: ',-4,ANVERT,' ',QUIT)
        IF (QUIT) GOTO 1500
        READ (ANVERT,'(I4)',IOSTAT=IOS) NVERT
        IF (IOS.NE.0 .OR. ANVERT.EQ.' ' .OR. NVERT.LT.1) GOTO 950
      ENDIF
      RNVERT = FLOAT(NVERT)
C
 1000 CONTINUE
      CALL GETSTR(12,'No. Points in Vertical..: ',1,NUMPTS,
     $            '[Enter 1 or 2]',QUIT)
      IF (QUIT) GOTO 1500
      IF (NUMPTS.EQ.' ') GOTO 1000
      IF (INDEX('12',NUMPTS).EQ.0) GOTO 1000
C
      CALL MSGPOS(18,19,BOLD//'W) Waded measurement')
      CALL MSGPOS(19,19,
     $            'S) Suspended measurement (e.g. bridge, cable, boat)')
      CALL MSGPOS(20,19,'A) Acoustic measurement'//OFF)
 1100 CONTINUE
      CALL GETSTR(13,'Measurement Type........: ',-1,TYPE,
     $            '[Enter: W, S, or A]',QUIT)
      IF (QUIT) GOTO 1500
      IF (TYPE.EQ.' ') GOTO 1100
      CALL UPCASE(TYPE)
      IF (INDEX('WSA',TYPE).EQ.0) GOTO 1100
      CALL ERSMSG()
C
 1200 CONTINUE
      CALL MSGPOS(18,18,BOLD//'A) Stable - even, firm, smooth')
      CALL MSGPOS(19,18,'B) Soft - silt, mud, muck')
      IF (TYPE.EQ.'W') THEN
        CALL MSGPOS(20,18,'C) Stable - uneven: gravel, cobbles')
      ELSE
        CALL MSGPOS(20,18,'C) Stable - uneven: cobbles, boulders')
      ENDIF
      CALL MSGPOS(21,18,'D) Unstable - shifting sand, dunes')
      IF (TYPE.EQ.'S') THEN
        CALL MSGPOS(22,18,
     $              'E) Stable - high velocity, some vertical angles')
        CALL MSGPOS(23,18,
     $              'F) Unstable - high velocity, some vertical angles'
     $              //OFF)
      ENDIF
      CALL MSGPOS(14,1,';')
      IF (TYPE.EQ.'S') THEN
        PMSG = '[Select A, B, C, D, E or F]'
      ELSE
        PMSG = '[Select A, B, C, or D]'
      ENDIF
      CALL GETSTR(14,'Channel Bed Type........: ',-1,BEDTYP,
     $            PMSG(:LENSTR(PMSG)),QUIT)
      CALL ERSMSG()
      IF (QUIT) GOTO 1500
      IF (BEDTYP.EQ.' ') GOTO 1200
      CALL UPCASE(BEDTYP)
      IF (INDEX('ABCDEF',BEDTYP).EQ.0) GOTO 1200
      IF (TYPE.EQ.'W' .AND. (BEDTYP.EQ.'E'.OR.BEDTYP.EQ.'F')) THEN
        CALL ERRMSG('NO ''E'' OR ''F'' BED TYPES WITH WADING MEAS.',
     $              .FALSE.)
        GOTO 1100
      ELSEIF (TYPE.EQ.'A' .AND. (BEDTYP.EQ.'E'.OR.BEDTYP.EQ.'F')) THEN
        CALL ERRMSG('NO ''E'' OR ''F'' BED TYPES WITH ACOUSTIC MEAS.',
     $              .FALSE.)
        GOTO 1100
      ENDIF
C
C--CHECK DEPTH AGAINST MEAS. TYPE
      IF (TYPE.EQ.'W' .AND. (AREA/WIDTH).GT.4.) THEN
        CALL ERSMSG()
        CALL MSGPOS(18,2,CHAR(7)//
     $          '** The mean depth for this measurement is (in feet): ;'
     $          )
        PRINT '(1X,F6.2)', AREA/WIDTH
        CALL MSGPOS(19,2,'** Please double check your answers for '//
     $              'WIDTH, AREA, and measurement TYPE')
      ENDIF
C
C--GOT ALL ANSWERS; NOW ALLOW GOING BACK AND CHANGING IF NEDDED
 1300 CONTINUE
      AOK = 'Y'
      CALL GETSTR(21,'Are the Above Answers Correct? ',1,AOK,
     $            '[Please answer (Y)es or (N)o]',QUIT)
      CALL ERSMSG()
      IF (INDEX('Nn',AOK).GT.0 .OR. QUIT) THEN
        REDO = .TRUE.
        PRVSTA = ' '
        GOTO 300
      ENDIF
      IF (INDEX('Yy',AOK).LE.0) GOTO 1300
C
C
C--------------------------------------------------------------
C--GOT ALL THE NECESSARY INFORMATION, NOW DO THE COMPUTATIONS--
C--------------------------------------------------------------
C--MEAN DEPTH (DM)
      DM = AREA/WIDTH
C
C--DETERMINE STD. ERROR FOR DEPTH (Sd)
      IF (TYPE.EQ.'W') THEN
        IF (BEDTYP.EQ.'A') SD = 2.
        IF (BEDTYP.EQ.'B') SD = (2.*(1.+SQRT(5./(2.*DM)**2)))
        IF (BEDTYP.EQ.'C') SD = (2.*(1.+SQRT(10./(2.*DM)**2)))
        IF (BEDTYP.EQ.'D') SD = 10.
      ELSEIF (TYPE.EQ.'S') THEN
        IF (BEDTYP.EQ.'A') SD = 2.
        IF (BEDTYP.EQ.'B' .OR. BEDTYP.EQ.'C')
     $      SD = (2.*(1.+SQRT(30./(2.*DM)**2)))
        IF (BEDTYP.EQ.'D') SD = 10.
        IF (BEDTYP.EQ.'E') SD = 5.
        IF (BEDTYP.EQ.'F') SD = 15.
      ELSEIF (TYPE.EQ.'A') THEN
        IF (BEDTYP.EQ.'A') SD = 2.
        IF (BEDTYP.EQ.'B' .OR. BEDTYP.EQ.'C')
     $      SD = (2.*(1.+SQRT(30./(2.*DM)**2)))
        IF (BEDTYP.EQ.'D') SD = 10.
      ENDIF
C
C--DETERMINE STD. ERROR FOR INSTRUMENT, BY VELOCITY (Si)
      IF (METER.EQ.'AASTD ') THEN
        IF (MEANV.GE.0.1 .AND. MEANV.LT.2.3) THEN
          SI = .7/MEANV
        ELSEIF (MEANV.LT.0.1) THEN
          CALL ERRMSG('MEAN VELOCITY BELOW COMPUTATION LIMITS',.TRUE.)
          GOTO 200
        ELSE
          SI = .3
        ENDIF
      ELSEIF (METER.EQ.'PYGSTD') THEN
        IF (MEANV.GE..1 .AND. MEANV.LT..5) THEN
          SI = .9*MEANV**(-1.3)
        ELSEIF (MEANV.LT.0.1) THEN
          CALL ERRMSG('MEAN VELOCITY BELOW COMPUTATION LIMITS',.TRUE.)
          GOTO 200
        ELSE
          SI = 1.8*MEANV**(-.3)
        ENDIF
        IF (MEANV.GT.3.0) THEN
          FNOTE = '*'
          VWARN = .TRUE.
        ENDIF
      ELSE
C--METER = PYGIND
        IF (MEANV.GE..1 .AND. MEANV.LT..5) THEN
          SI = .6*MEANV**(-1.4)
        ELSEIF (MEANV.LT.0.1) THEN
          CALL ERRMSG('MEAN VELOCITY BELOW COMPUTATION LIMITS',.TRUE.)
          GOTO 200
        ELSE
          SI = 1.05*MEANV**(-.6)
        ENDIF
        IF (MEANV.GT.3.0) THEN
          FNOTE = '*'
          VWARN = .TRUE.
        ENDIF
      ENDIF
C
C--DETERMINE STD. ERROR FOR PULSATION (St) AND VERT. DIST. (Ss)
      IF (NUMPTS.EQ.'1') THEN
        ST = (16.6*TIOBS**(-.28))
        SS = SQRT((120.4/RNVERT)+5.02)
      ELSEIF (NUMPTS.EQ.'2') THEN
        ST = (16.0*TIOBS**(-.36))
        SS = SQRT((17.75/RNVERT)+0.74)
      ENDIF
C
C--DETERMINE STD. ERROR FOR HORIZ. ANGLES (Sh)
      IF (HORIZA) THEN
        SH = 1
      ELSE
        SH = 0
      ENDIF
C
C--SET STD. ERROR FOR HORIZ. DIST. OF DEPTH AND VELOCITY (Sv)
      SV = 32.*RNVERT**(-.88)
C
C--CALCULATE "SQ" BASED ON ALL THE ABOVE PARAMETERS
      SQ = SQRT(((SD**2+ST**2)/RNVERT)+SI**2+SS**2+SH**2+SV**2+SBSDSV)
C      SQ=(SD**2 + SI**2 + ST**2 + SS**2 + SH**2 + SV**2 + SBSDSV)**.5
C
C--MEASUREMENT ACCURACY CLASSES
      IF (SQ.LE.2.0) THEN
        MACCY = '(excellent)'
      ELSEIF (SQ.LE.5.0) THEN
        MACCY = '(good)'
      ELSEIF (SQ.LE.8.0) THEN
        MACCY = '(fair)'
      ELSE
        MACCY = '(poor)'
      ENDIF
C
C--OUTPUT THE RESULTS
      IF (OUTWAT) THEN
        WRITE (OUN,9035) FNOTE, AMESNO, METER, NVERT, NUMPTS, TYPE, 
     $                   TIOBS, MEANV, DM, AQ, SQ, MACCY
      ELSE
        WRITE (OUN,9040) FNOTE, AMESNO, METER, NVERT, NUMPTS, TYPE, 
     $                   TIOBS, MEANV, DM, SQ, MACCY
      ENDIF
      WOUT = .TRUE.
      WRITE (PMSG,9045) BOLD, 'Percent Uncertainty = ', SQ, MACCY, OFF
      CALL MSGPOS(17,22,PMSG(1:48))
C
C--BOTTOM OF LOOP; GO BACK FOR MORE
 1400 CONTINUE
      AOK = 'Y'
      CALL GETSTR(21,'Next Measurement? ',1,AOK,
     $            '[Please answer (Y)es or (N)o]',QUIT)
      IF (INDEX('Yy',AOK).GT.0) GOTO 200
      IF (.NOT.(INDEX('Nn',AOK).GT.0.OR.QUIT)) GOTO 1400
C
 1500 CONTINUE
      IF (VWARN) WRITE (OUN,'(/A)') 
     $                 '* Mean velocity for pygmy meter exceeds 3.0 fps'
      CALL MSGPOS(23,1,';')
      IF (OWFILE.NE.' ') CLOSE (IUN)
      IF (WOUT) THEN
        PRINT *, '** Output file is: ''', OUTFIL, ''''
        CLOSE (OUN)
      ELSE
        CLOSE (OUN,STATUS='DELETE',IOSTAT=IOS)
      ENDIF
      STOP
 9005 FORMAT (1X,A,'MEASERR - MEASUREMENT UNCERTAINTY PROGRAM ',A,19X,
     $        '[Version ',A,']',//,1X,
     $ 'This program determines the statistical measurement uncertainty'
     $ ,' for',/,
     $ ' wading, suspended, and acoustic discharge measurements.',/,
     $ ' The calculations are based on USGS OPEN-FILE REPORT 92-144',
     $ ' (Sauer & Meyer).',//,' NOTE: If necessary, you may enter ',A,A,
     $ A,' at most prompts',/,7X,'to exit the program at any time.',//,
     $ ' If you want to use discharge-measurement data from an ',
     $ 'OUTWAT retrieval')
 9010 FORMAT (A6,I2.2,A4)
 9015 FORMAT (T8,A15,T25,A5,T62,A6,T74,A9,T88,A5,T127,A10,T226,A4)
 9020 FORMAT (/A,' - ',A)
 9025 FORMAT (' MEAS.',2X,' METER',4X,'NO.',5X,'NO.',3X,'MEAS',9X,
     $        'MEAN',3X,'MEAN',6X,'MEAS.',5X,'UNCER-'/'  NO. ',2X,
     $        ' TYPE',3X,'VERTS.',2X,'POINTS',2X,'TYPE',2X,'TIME',3X,
     $        'VEL.',3X,'DEPTH',6X,'Q',8X,'TAINTY '/' -----',2X,
     $        '------',2X,'------',2X,'------',2X,'----',2X,'----',2X,
     $        '------',2X,'------',2X,'----------',2X,'-------')
 9030 FORMAT (' MEAS.',2X,' METER',4X,'NO.',5X,'NO.',3X,'MEAS',9X,
     $        'MEAN',3X,'MEAN',4X,'UNCER-'/'  NO. ',2X,' TYPE',3X,
     $        'VERTS.',2X,'POINTS',2X,'TYPE',2X,'TIME',3X,'VEL.',3X,
     $        'DEPTH',3X,'TAINTY '/' -----',2X,'------',2X,'------',2X,
     $        '------',2X,'----',2X,'----',2X,'------',2X,'------',2X,
     $        '-------')
 9035 FORMAT (A1,A5,2X,A6,3X,I4,6X,A1,5X,A1,4X,I3,2X,F6.2,2X,F6.2,3X,
     $        A10,2X,F5.1,'%',2X,A)
 9040 FORMAT (A1,A5,2X,A6,3X,I4,6X,A1,5X,A1,4X,I3,2X,F6.2,2X,F6.2,3X,
     $        F5.1,'%',2X,A)
 9045 FORMAT (A4,A22,F4.1,1X,A12,A4)
      END
C
C**SHOW INPUT FIELD AND PROMPT FOR INPUT  *****************************
      SUBROUTINE GETSTR(ROW,PROMPT,LFLD,ANSWER,MSG,QUIT)
C
      INTEGER J, LENSTR, ROW, LFLD, IROW
      CHARACTER PROMPT*(*), ANSWER*(*), ANS*80, MSG*(*)
      LOGICAL QUIT
C
      CHARACTER BAKSTR*80, FLDSTR*80, STRNG*120
      CHARACTER EL*4, BOLD*4, REV*4, OFF*4
      INTEGER LANS, N
C
C--ANSI CONTROL FUNCTIONS
      EL = CHAR(27)//'[0K'
      BOLD = CHAR(27)//'[1m'
      REV = CHAR(27)//'[7m'
      OFF = CHAR(27)//'[0m'
C--INIT
      QUIT = .FALSE.
      LANS = LENSTR(ANSWER)
      IROW = ROW
C
C--SET HORIZ./VERT. CURSOR-POSITION SEQUENCE (ANSI)
      IF (IROW.GT.23) IROW = 23
C
C--STRING OF <BACKSPACES> AND UNDERSCORES FOR FIELD DISPLAY
      DO 100 J = 1, LANS
        BAKSTR(J:J) = CHAR(8)
        FLDSTR(J:J) = ANSWER(J:J)
 100  CONTINUE
      DO 200 J = LANS + 1, 80
        BAKSTR(J:J) = CHAR(8)
        FLDSTR(J:J) = '_'
 200  CONTINUE
C
C--DISPLAY MESSAGE (IF ANY)
 300  CONTINUE
      STRNG = BOLD//MSG//OFF
      IF (MSG.NE.' ') CALL MSGPOS(IROW,80-LENSTR(MSG)+1,STRNG)
C--DISPLAY THE PROMPT
      N = IABS(LFLD)
      STRNG = BOLD//PROMPT//REV//FLDSTR(1:N)//BAKSTR(1:N)//';'
      CALL MSGPOS(IROW,2,STRNG)
C--GET VALID ANSWER
      READ (*,'(A)') ANS
      CALL MSGPOS(IROW,2,OFF)
      LANS = LENSTR(ANS)
      IF (LANS.NE.0) THEN
        IF (INDEX('QUIT quit Quit ',ANS(1:5)).GT.0) THEN
          QUIT = .TRUE.
          GOTO 400
        ELSE
          ANSWER = ANS
        ENDIF
C
C--IF LFLD IS NEG(-) THEN ANSWER LENGTH CAN BE .LE. LFLD.
C--BUT, IF LFLD IS POS(+) THEN ANSWER LENGTH MUST BE .EQ. LFLD
        IF ((LFLD.GT.0) .AND. (LANS.NE.LFLD)) GOTO 300
        IF (LANS.GT.ABS(LFLD)) GOTO 300
      ENDIF
C
 400  CONTINUE
      STRNG = OFF//PROMPT//ANSWER//EL//';'
      CALL MSGPOS(IROW,2,STRNG)
      RETURN
 9005 FORMAT (A1,A1,I2.2,A3)
 9010 FORMAT (1X,A,A)
      END
C
C*** CLEAR SCREEN USING ANSI CLEAR AND HOME SEQUENCES ******************
      SUBROUTINE CLS()
C
      CHARACTER*1 ESC, FF
      CHARACTER*8 CLINE
C
      ESC = CHAR(27)
C--USE FORM FEED FOR SOME TERMINALS, ANSI CONTROL CHARS. FOR OTHERS
      FF = CHAR(12)
      CLINE = FF//ESC//'[2J'//ESC//'[f'
      PRINT *, CLINE
      RETURN
      END
C
C** CONVERT INPUT STRING TO UPPERCASE ********************************
      SUBROUTINE UPCASE(CHSTR)
C
      INTEGER J, LENSTR, N
      CHARACTER CHSTR*(*), CH*1
C
C--PARSE INPUT CHSTR FOR LOWERCASE LETTERS AND CONVERT TO UPPER
      N = LENSTR(CHSTR)
      DO 100 J = 1, N
        CH = CHSTR(J:J)
        IF (CH.GE.'a' .AND. CH.LE.'z') CHSTR(J:J)
     $      = CHAR(ICHAR(CH)-(ICHAR('a')-ICHAR('A')))
 100  CONTINUE
C
      RETURN
      END
C
C *** GET LENGTH OF CHARACTER STRING *********************************
      INTEGER FUNCTION LENSTR(STRING)
C
      CHARACTER*(*) STRING
      CHARACTER*1 DELIM
      INTEGER MAX, I
      PARAMETER (DELIM=' ')
C
      MAX = LEN(STRING)
      DO 100 I = MAX, 1, -1
        IF (STRING(I:I).NE.DELIM) THEN
          LENSTR = I
          GOTO 200
        ENDIF
 100  CONTINUE
      LENSTR = 0
C
 200  CONTINUE
      RETURN
      END
C
C** CONVERT NUMERIC STRING TO REAL VALUE *******************************
      REAL FUNCTION VALSTR(ISTR)
C
      CHARACTER*(*) ISTR
      INTEGER N
      CHARACTER*8 FMT
C
      N = LEN(ISTR)
      WRITE (FMT,9005) N
C
C     change from unformatted read to formatted read since the
C     unformatted read of internal file is not part of ANSI f77.
      READ (ISTR,FMT,ERR=100) VALSTR
      GOTO 200
C
 100  CONTINUE
      VALSTR = 0.0
 200  CONTINUE
      RETURN
 9005 FORMAT ('(F',I3,'.0)')
      END
C
C** CHECK THAT CHARACTERS ENTERED ARE ALL NUMBERS ********************
      LOGICAL FUNCTION NUMBER(NUMSTR)
C
      CHARACTER NUMSTR*(*), CH*1
      INTEGER LENSTR, K, L
C
      L = LENSTR(NUMSTR)
      NUMBER = .TRUE.
      CH = NUMSTR(1:1)
      IF (((CH.LT.'0').OR.(CH.GT.'9')) .AND. 
     $    ((CH.NE.'+').AND.(CH.NE.'-').AND.(CH.NE.'.'))) THEN
        NUMBER = .FALSE.
        GOTO 200
      ENDIF
      DO 100 K = 2, L
        CH = NUMSTR(K:K)
        IF ((CH.LT.'0') .OR. (CH.GT.'9') .AND. (CH.NE.'.')) THEN
          NUMBER = .FALSE.
          GOTO 200
        ENDIF
 100  CONTINUE
C
 200  CONTINUE
      RETURN
      END
C
C** PRINT AT SPECIFIED POSITION ON ANSI TERMINAL **********************
      SUBROUTINE MSGPOS(ROW,COL,MSG)
C--PRINT MSG AT SPECIFIED
C--LOCATION ON SCREEN. USE ;
C--AS LAST CHARACTER OF MSG FOR
C--NO CR/LF AT END OF LINE.
C
      INTEGER IROW, ROW, ICOL, COL, LM, LENSTR
      CHARACTER ESC*1, MSG*(*)
      CHARACTER ESCSEQ*8
C
C--ANSI ESCAPE SEQUENCE FOR CURSOR POSITIONING IS: "<ESC>[ROW;COLf"
      ESC = CHAR(27)
      LM = LENSTR(MSG)
C--SET LM TO LENGTH OF INPUT STRING IF STRING CONTAINS ALL BLANKS
      IF (LM.LT.1) LM = LEN(MSG)
      IROW = ROW
      IF (IROW.GT.24) IROW = 24
      ICOL = COL
      IF (ICOL.GT.80) ICOL = 80
      WRITE (ESCSEQ,9005) ESC, '[', IROW, ';', ICOL, 'f'
C
      IF (MSG(LM:LM).NE.';') THEN
        PRINT 9010, ESCSEQ, MSG(:LM)
      ELSEIF (LM.GT.1) THEN
        CALL NCRLF(ESCSEQ//MSG(:LM-1))
      ELSE
        CALL NCRLF(ESCSEQ)
      ENDIF
C
      RETURN
 9005 FORMAT (A1,A1,I2.2,A1,I2.2,A1,A)
 9010 FORMAT (1X,A8,A)
      END
C
C** PRINT ERROR MESSAGE AND ALLOW EXIT ********************************
      SUBROUTINE ERRMSG(MSG,FATAL)
C
      CHARACTER MSG*(*)
      LOGICAL FATAL
      CHARACTER CTNU*1
      CHARACTER BOLD*4, OFF*4
      INTEGER COL, LENSTR
C
      BOLD = CHAR(27)//'[1m'
      OFF = CHAR(27)//'[0m'
      COL = ((80-LENSTR(MSG))/2) - 3
C
C--PRINT MESSAGE AT BOTTOM OF SCREEN
      CALL ERSMSG()
      CALL MSGPOS(19,COL,BOLD//'** '//MSG(1:LENSTR(MSG))//' **'//OFF)
      IF (FATAL) THEN
        CALL MSGPOS(20,22,'This measurement cannot be completed.')
      ELSE
        CALL MSGPOS(20,22,'Enter an appropriate answer or ''QUIT''')
      ENDIF
      CALL MSGPOS(21,27,'Press <CR> to continue...;')
      READ (*,'(A)',ERR=100) CTNU
C
C--ERASE MESSAGE
 100  CONTINUE
      CALL ERSMSG()
C
      RETURN
      END
C
C*** PRINT CHAR STRING W/O A CR/LF *************************************
      SUBROUTINE NCRLF(OUTSTR)
C--TNOUA IS A PRIME SYSTEM ROUTINE
C
      CHARACTER OUTSTR*(*)
      CHARACTER*132 RECD
      INTEGER LSTR, LENSTR
      INTEGER*2 I2LSTR
C
      RECD = ' '//OUTSTR
      LSTR = LENSTR(RECD)
C--SET LSTR TO LENGTH OF INPUT STRING IF STRING CONTAINS ALL BLANKS
      IF (LSTR.LT.1) LSTR = LEN(OUTSTR)
      I2LSTR = LSTR
      CALL TNOUA(RECD,I2LSTR)
      RETURN
      END
C
C** ERASE MESSAGE AREA OF SCREEN (ROWS 18+) **************************
      SUBROUTINE ERSMSG()
C
      CHARACTER*80 RECD
      RECD = ' '
C
C     On Prime computers trailing blanks are deleted prior to sending to
C     standard out, so the following line is used on Prime systems.  The
C     other lines send a blank line to the screen to "erase" the message
C     area, this is needed on PC's which may erase the whole screen with
C     the escape sequence used below.  Note, either senario works on the
C     Data General AViiON's.
C     CALL MSGPOS(18,2,CHAR(27)//'[J')
      CALL MSGPOS(18,1,RECD)
      CALL MSGPOS(19,1,RECD)
      CALL MSGPOS(20,1,RECD)
      CALL MSGPOS(21,1,RECD)
      CALL MSGPOS(22,1,RECD)
      CALL MSGPOS(23,1,RECD)
      RETURN
      END
