#!/bin/sh
# usr/opt/wrdapp/measerr2.5/test/test.sh -- run MEASERR test data sets
#
# Usage: test.sh [start [stop]]
#        test.sh [start [stop] | tee test.out
#
#        where: start = starting test number
#                stop = ending test number (may be same as start)
#
# To use this script for another program in a different directory, globally
# change the program name (be sure to change both upper and lower case
# occurrences) and set the default value for the Stop variable as appropriate.
#
# History: 12/24/91, mblalock, initial coding
#          04/27/92, mygoze, restructuring
#          02/01/93, rsregan, modified for use with measerr
#


#
# Variable definitions
# --------------------
#
  TOPDIR=..
  DATA=$TOPDIR/data
  CHECK=$TOPDIR/test/check.sh
  CLEAN=$TOPDIR/test/clean.sh
  PROG=$TOPDIR/bin/measerr
  PROGNM=MEASERR
  DIVD=========================================

#
#
  exec 2>&1                                # stderr shows up in .out file

  Start=${1:-1}                            # by default, start at 1
  Stop=${2:-1}                             # by default, stop at 1
  if [ $Start -lt 1 ] ; then Start=1 ; fi
  if [ $Stop -lt 1 ] ; then Stop=3 ; fi
  if [ $Start -gt 3 -o $Stop -gt 3 ]
  then
    echo
    echo "Warning, invalid arguments--test range is 1 - 3 for $PROGNM"
    echo "input arguments were $Start - $Stop"
    echo
    if [ $Stop -gt 3 ] ; then Stop=3 ; fi
    if [ $Start -gt 3 ]
    then
      echo "no $PROGNM tests will be performed"
    else
      echo "Tests $Start - $Stop will be performed"
    fi
    echo
  fi

#
# begin test runs
#
  Test=$Start

  if [ $Test -ge $Start -a $Test -le $Stop ]
  then
    echo
    echo $DIVD$DIVD
    echo "Begin processing $PROGNM test runs $Start to $Stop"
    echo
    date
  fi
#
# remove old output file
#
  $CLEAN

  while [ $Test -ge $Start -a $Test -le $Stop ]
  do
    echo
    echo
    echo $DIVD$DIVD
    echo "Test run number $Test"
    Name=measer0$Test
    rm -f go
    echo
    echo $DATA/$Name > go
    cat $DATA/go$Test >> go

    $PROG < go
    Test=`expr $Test + 1`
  done
#
  if [ $Test -ge $Start -a $Test -le $Stop ]
  then
    echo
    echo
    echo $DIVD$DIVD
    echo "Completed $PROGNM test runs $Start to $Stop"
    echo
  fi

# check output against original output in $DATA directory
  $CHECK measer
